clear
close all
clc

% output: Ifin1, Ifin2, Vfin

% this script generates totmatfiles files .mat. Each file deals with 100 particle
% image pairs of the same reference field
tic
totmatfiles=1; % number of different fields

for test1=1:totmatfiles
extrapx=4; % number of pixels beyond every boundary (particles are allowed to leave/enter the camera field of view)

% resx=resy must be set (r0)
resx=32; % image resolution [pixels]
resy=32; % for larger images (es 1280x1024): it is better to divide them
% into smaller images (es. 32x32) before applying neural network algorithm.
% Smaller structures can be identified in this way.
% The size of the input images of the neural network is similar
% to the size of the interrogation window in cross-correlation method.

xmax=2*extrapx+resx;
ymax=2*extrapx+resy;

xvf=-xmax/2+0.5:1:xmax/2-0.5;%1:1:xmax; % pixels along x (x axis: sx->dx)
yvf=ymax/2-0.5:-1:-ymax/2+0.5;%1:1:ymax; % pixels along y (y axis: down->top)

n_imagepairs=1000; % image pairs for each different field
um='noriscala';
%um='non_riscalare';

Ifin1=zeros(n_imagepairs,resx,resy);
Ifin2=zeros(n_imagepairs,resx,resy);
%Vfin=zeros(n_imagepairs,6);
Vfint=zeros(n_imagepairs,14);

u0=(-4+8*rand); % [pixels/frame] -4 - 4
v0=(-4+8*rand);
Ju=(-0.05+0.1*rand(1,2));%(-0.05+0.1*rand(1,2)); % [1/frame]
Jv=(-0.05+0.1*rand(1,2));%(-0.05+0.1*rand(1,2));
Hu=(-0.001+0.002*rand(2,2));%(-0.001 + 0.002*rand(2,2)); % [1/(pixels*frame)] -0.001 - 0.001
Hv=(-0.001+0.002*rand(2,2));%(-0.001 + 0.002*rand(2,2)); % [1/(pixels*frame)] -0.001 - 0.001

for nip=1:n_imagepairs

I1=zeros(xmax,ymax);
I2=zeros(xmax,ymax);

% LOCATION OF PARTICLE CENTRES
rho=0.029;%156;%(0.0122 + 0.0061*rand); %[ppp] %(0.05 + (0.1-0.05)*rand(1)); 0.05 - 0.1 (2 art.)
n_particles=round(rho*xmax*ymax);%round(200+100*rand);%; 20 % number of particles

r0=xmax/2*(-1 + 2.*rand(n_particles,2));% initial particle centres (x0, y0)[pixel], (xmax=2*extrapx+resx)

% 2D VELOCITY FIELD
%u0=(-4+8*rand); % [pixels/frame] -4 - 4
%v0=(-4+8*rand);
%u0=0;
%v0=0;

%Ju=zeros(1,2);
%Jv=zeros(1,2);
%Ju=(-0.05+0.1*rand(1,2));%(-0.05+0.1*rand(1,2)); % [1/frame]
%Jv=(-0.05+0.1*rand(1,2));%(-0.05+0.1*rand(1,2));

%Hu=zeros(2,2);
%Hv=zeros(2,2);
%Hu=(-0.001+0.002*rand(2,2));%(-0.001 + 0.002*rand(2,2)); % [1/(pixels*frame)] -0.001 - 0.001
%Hv=(-0.001+0.002*rand(2,2));%(-0.001 + 0.002*rand(2,2)); % [1/(pixels*frame)] -0.001 - 0.001

% round
dec=5;

u0=round(u0,dec);
v0=round(v0,dec);
Ju=round(Ju,dec);
Jv=round(Jv,dec);
Hu=round(Hu,dec);
Hv=round(Hv,dec);


uu= @(xx,yy) u0+Ju*[xx;yy]+[xx,yy]*Hu*[xx;yy];
vv= @(xx,yy) v0+Jv*[xx;yy]+[xx,yy]*Hv*[xx;yy];


%Vfin(nip,:)=[u0, v0, Ju, Jv]; % training
Vfint(nip,:)=[u0, v0, Ju, Jv Hu(1,1) Hu(1,2) Hu(2,1) Hu(2,2) Hv(1,1) Hv(1,2) Hv(2,1) Hv(2,2)]; % test



% INTEGRATION OVER TIME (multiplication by time interval)

h=0.5; % half time step


%I0=(200+55*rand(n_particles,1)); % peak intensity % [grey value] 200 - 255
I0=randi(56,[n_particles,1])+199;

dp=1.5+2.5*rand(n_particles,1);%0.5+1.5*rand(n_particles,1);% effective particle image diameter [pixel] 1.5 - 3.5

for np=1:n_particles
    
    % particle centres, first exposition
    rout1u=r0(np,1)-h*uu(r0(np,1),r0(np,2)); %horizontal component
    rout1v=r0(np,2)-h*vv(r0(np,1),r0(np,2)); %vertical component

    % particle centres, second exposition
    rout2u=r0(np,1)+h*uu(r0(np,1),r0(np,2)); %horizontal component
    rout2v=r0(np,2)+h*vv(r0(np,1),r0(np,2)); %vertical component

for i=1:length(yvf)
     for j=1:length(xvf)
 I1(i,j)=I1(i,j)+I0(np)*exp((-(xvf(j)-rout1u)^2-(yvf(i)-rout1v)^2)/(1/8*dp(np)^2));
 I2(i,j)=I2(i,j)+I0(np)*exp((-(xvf(j)-rout2u)^2-(yvf(i)-rout2v)^2)/(1/8*dp(np)^2));
     end
end
end


I1=imcrop(I1,[extrapx+1,extrapx+1,resx-1,resy-1]); % [XMIN YMIN WIDTH HEIGHT]
I2=imcrop(I2,[extrapx+1,extrapx+1,resx-1,resy-1]); % [XMIN YMIN WIDTH HEIGHT]

I1(I1>255)=255;
I2(I2>255)=255;
div_f=255; % con div=255 porto le immagini tra 0 e 1
I1=round(I1/div_f,dec);
I2=round(I2/div_f,dec);

noise1=imnoise(I1,'gaussian',0,max(max(I1))/100);
noise2=imnoise(I2,'gaussian',0,max(max(I2))/100);
I1=I1+noise1;
I2=I2+noise2;




Ifin1(nip,:,:)=I1(:,:);
Ifin2(nip,:,:)=I2(:,:);

% figure
% imshow(I1, 'initialmagnification',1000)
% figure
% imshow(I2,'initialmagnification',1000)
% 

end


%% SAVING

str_imp=num2str(n_imagepairs);

if noise1==zeros(xmax,ymax)
str_noise='nonoise';
else
    str_noise='withnoise';
end

str_type='manuale';

if strcmp(um,'riscala')==1
    Vfin(:,1:2)=round(Vfin(:,1:2)/100,dec);
    um='umu';
else
    um='umd';
end

%%
name=sprintf('ig3_%s_%s_%s_rhocost_div%d_dec%d_%s_uguali_%d', str_imp, str_noise, str_type, div_f, dec, um, test1);
%name='data'

save(name, 'Ifin1', 'Ifin2', 'Vfint')

%sound(sin(1:3000));

%%
% 
% uq=zeros(32,32);
% vq=zeros(32,32);
% 
% xpq=-15.5:1:15.5;
% ypq=15.5:-1:-15.5;
% for i=1:length(xpq)
%     for j=1:length(ypq)
% uq(j,i)=uu(xpq(i),ypq(j));
% vq(j,i)=vv(xpq(i),ypq(j));
%     end
% end
% 
% figure
% quiver(xpq, ypq, uq, vq)
% xlabel('u [pixels/frame]')
% ylabel('v [pixels frame]')
% axis([min(xpq),max(xpq),min(ypq), max(ypq)])
% x=-49.5:49.5;
% y=-49.5:49.5;
% I0=250;
% x0=0;
% y0=0;


end
toc